import 'package:datadome_flutter_dio/datadome_dio.dart';
import 'package:datadome_flutter_dio/datadome_logger.dart';
import 'package:datadome_flutter_dio/retriable_request.dart';
import 'package:dio/dio.dart';
import 'package:flutter/material.dart';
import 'captcha_page.dart';
import 'package:shared_preferences/shared_preferences.dart';


class DataDomeInterceptor extends Interceptor {
  String? dataDomeKey;
  Dio? dio;
  late BuildContext context;
  static List<RetriableRequest> failedRequests = [];

  DataDomeInterceptor(String dataDomeKey, Dio dio, BuildContext context) {
    this.dataDomeKey = dataDomeKey;
    this.dio = dio;
    this.context = context;
  }

  @override
  void onRequest(RequestOptions options, RequestInterceptorHandler handler) async {
    DataDomeLogger.info('Validating request ${options.uri}');

    final prefs = await SharedPreferences.getInstance();
    String cookie = prefs.getString('co.datadome.cookie')?? '';
    if (cookie.isNotEmpty) {
      options.headers['Cookie'] = cookie;
    }

    return handler.next(options);
  }

  @override
  void onResponse(Response response, ResponseInterceptorHandler handler) {
    super.onResponse(response, handler);
  }

  @override
  void onError(DioError err, ErrorInterceptorHandler handler) {
    var ddHeaders = err.response!.headers.map.containsKey('x-dd-b');
    var code = err.response!.statusCode;
    if ((code == 401 || code == 403) && ddHeaders) {
      //blocked by datadome
      failedRequests.add(RetriableRequest(dio, err, handler));
      if (CaptchaPage.isDisplayed) {
        return;
      }

      var captchaUrl = err.response!.data['url'];
      showCaptcha(
          context,
          captchaUrl,
              (String cookie) {
            didResolveCaptcha(cookie);
          }
      );
      return;
    }

    super.onError(err, handler);
  }

  late BuildContext dialogContext;
  void showCaptcha(BuildContext context, String? captchaUrl, ValueChanged<String> onCaptchaResolved) {
    DataDomeLogger.info('Presenting the captcha');

    CaptchaPage.isDisplayed = true;
    showGeneralDialog(
      context: context,
      barrierDismissible: false,
      pageBuilder: (context, __, ___) {
        dialogContext = context;
        return new CaptchaPage(captchaUrl: captchaUrl, onCaptchaResolved: onCaptchaResolved);
      },
    );
  }

  void dismissCaptcha() {
    DataDomeLogger.info('Dismissing the captcha');

    Navigator.pop(dialogContext);
    CaptchaPage.isDisplayed = false;
  }

  void didResolveCaptcha(String cookie) {
    DataDomeLogger.info("Did resolved captcha with cookie $cookie");

    DataDomeDio.storeCookie(cookie);
    dismissCaptcha();

    DataDomeLogger.info("Retrying all failed requests");
    retryFailedRequests();
  }

  void retryFailedRequests() {
    for (var retriable in failedRequests) {
      retry(retriable);
    }

    failedRequests.clear();
  }

  Future<Null> retry(RetriableRequest retriable) async {
    RequestOptions requestOptions = retriable.error.requestOptions;
    final options = new Options(
      method: requestOptions.method,
      headers: requestOptions.headers,
    );

    Response response = await retriable.dio!.request(
      requestOptions.path,
      cancelToken: requestOptions.cancelToken,
      data: requestOptions.data,
      onReceiveProgress: requestOptions.onReceiveProgress,
      onSendProgress: requestOptions.onSendProgress,
      queryParameters: requestOptions.queryParameters,
      options: options,
    );

    retriable.handler.resolve(response);
  }
}
